/**   \file  check_modmgr.c
 *   \brief  Check unit testing framework main function for ModMgr
 *  \author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  \internal
 *       Created:  19/03/2022
 *      Revision:  none
 * Last modified:  2024-07-21 13:12
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "oom.h"                                /* oomtest_setup */
#include <check.h>
#include <stdlib.h>                             /* EXIT_SUCCESS */
#include <stdio.h>                              /* printf */
#include <stdlib.h>                             /* abort() */
#include "debug/memory.h"                       /* memreport */

Suite* sample_suite();
Suite* modmgr_searchpath_suite();
Suite* modmgr_modules_suite();

int main(int argc, char* argv[], char* env[])
{
    SRunner *sr;
    int number_failed;
    enum print_output ck_verbosity;

    (void) env; /* Avoids "unused" warning */

    /*
     * CK_SILENT
     * CK_MINIMAL
     * CK_NORMAL
     * CK_VERBOSE
     * CK_ENV
     */
    ck_verbosity = CK_ENV;
    printf("Verbosity: ");
    if ((argc==2)&&(0==strcmp(argv[1],"-v"))) {
        printf("yes-CK_VERBOSE\n");
        ck_verbosity = CK_VERBOSE;
    } else
        printf("default-CK_ENV (look for CK_VERBOSITY env)\n");

    /* Limit available RAM to really available RAM for all
     * suites/testcase/process/fork to test OOM without exhausting the system's
     * resources, hitting the swap or consumming too much time to fill the RAM
     * */
    oomtest_config(RAMLIMIT_HARD);

    /* Empty suite, to initialize the framework with the specific
     * function and to display a CRLF in the output */
    sr = srunner_create(suite_create(""));

    /*  CK_FORK
     *  CK_NOFORK
     *  CK_FORK_GETENV (default)
     */
    printf ("Fork mode (with signal catch): ");
    if ((getenv("CK_FORK")) && (strlen(getenv("CK_FORK"))>0)) {
        printf("%s if possible (CK_FORK defined)",getenv("CK_FORK"));
    } else {
        printf("yes if possible (default)");
    }
    printf("\n");
    srunner_set_fork_status (sr, CK_FORK_GETENV);

    /* Print what will be run, in case of environment variable override */
    printf("CK_RUN_SUITE=");
    if ((getenv("CK_RUN_SUITE")) && (strlen(getenv("CK_RUN_SUITE"))>0))
        printf("%s\n",getenv("CK_RUN_SUITE"));
    else
        printf("all\n");
    printf("CK_RUN_CASE=");
    if ((getenv("CK_RUN_CASE")) && (strlen(getenv("CK_RUN_CASE"))>0))
        printf("%s\n",getenv("CK_RUN_CASE"));
    else
        printf("all\n");
    printf("CK_INCLUDE_TAGS=");
    if ((getenv("CK_INCLUDE_TAGS")) && (strlen(getenv("CK_INCLUDE_TAGS"))>0))
        printf("%s\n",getenv("CK_INCLUDE_TAGS"));
    else
        printf("all\n");
    printf("CK_EXCLUDE_TAGS=");
    if ((getenv("CK_EXCLUDE_TAGS")) && (strlen(getenv("CK_EXCLUDE_TAGS"))>0))
        printf("%s\n",getenv("CK_EXCLUDE_TAGS"));
    else
        printf("all\n");

    srunner_add_suite (sr, sample_suite());
    srunner_add_suite (sr, modmgr_searchpath_suite());
    srunner_add_suite (sr, modmgr_modules_suite());

    srunner_run_all(sr, ck_verbosity);

    number_failed = srunner_ntests_failed(sr);
    srunner_free(sr);

    /* Output memory usage and leaks report */
    memreport();

    printf("\n");

    /*
     * Return error code 1 if the one of test failed.
     * Return 77 if test skipped
     */
    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}
